@@ -12,20 +12,26 @@ Key Features:
 # pylint: disable=wrong-import-position
 import ast
 import json
+import os
 import sys
+import traceback
 from concurrent.futures import ThreadPoolExecutor, as_completed
 from pathlib import Path
 
 import pyperclip  # type: ignore
 from openai import OpenAI
-from pydantic import BaseModel
 from rich import print
 from tqdm import tqdm  # type: ignore
-from wrapworks import cwdtoenv  # type: ignore
+from wrapworks import cwdtoenv
+
+from pythion.src.models.doc_writer_models import (  # type: ignore
+    ModuleDocString,
+    ObjDocString,
+)
 
 cwdtoenv()
 from pythion.src.indexer import NodeIndexer
-from pythion.src.models.core_models import SourceCode, SourceDoc
+from pythion.src.models.core_models import ModuleDocSave, SourceCode, SourceDoc
 from pythion.src.models.prompt_models import DOC_PROFILES
 
 
@@ -50,6 +56,7 @@ class DocManager:
         self.root_dir: str = root_dir
         self.cache_dir: str = ".pythion"
         self.doc_cache_file_name: str = "doc_cache.json"
+        self.module_cache_file_name: str = "module_cache.json"
 
         self.folders_to_ignore = [".venv", ".mypy_cache"]
         self.indexer = indexer or NodeIndexer(
@@ -62,17 +69,36 @@ class DocManager:
 
     def _make_cache_dir(self):
         """
-        Creates a cache directory if it does not exist.
+        Creates a cache directory and updates the .gitignore file if necessary.
 
-        This method checks the path defined in the `cache_dir` attribute and creates the specified directory along with any necessary parent directories. If the directory already exists, no action is taken.
+        This method ensures that the directory specified by `self.cache_dir` exists,
+        creating it if it does not. It also adds the directory to a .gitignore file in
+        the current working directory to prevent it from being tracked by Git.
 
         Attributes:
-            cache_dir (str): The directory path where the cache should be created.
+        - cache_dir: The path of the cache directory to be created.
+
+        Returns:
+        - None: This method does not return a value.
         """
 
         path = Path(self.cache_dir)
         path.mkdir(parents=True, exist_ok=True)
 
+        current_dir = Path.cwd()
+        gitignore_path = current_dir / ".gitignore"
+        pattern_to_add = f"\n{self.cache_dir}"
+
+        if gitignore_path.exists():
+            with open(gitignore_path, "r+", encoding="utf-8") as f:
+                content = f.read()
+                if self.cache_dir in content:
+                    return
+                f.write(pattern_to_add)
+        else:
+            with open(gitignore_path, "w", encoding="utf-8") as f:
+                f.write(pattern_to_add)
+
     def build_doc_cache(self, use_all: bool = False, dry: bool = False):
         """
         Builds a cache of docstrings for objects in the indexer.
@@ -139,16 +165,24 @@ class DocManager:
             "Docstring cache built successfully. Use iter-docs to go through the docstrings"
         )
 
-    def iter_docs(self):
+    def iter_docs(self, build: bool = False):
         """
-        Iterates through cached documentation strings, allowing the user to manage and save them.
+        Iterates through cached docstrings and manages clipboard operations.
 
-        This function reads documentation strings from a cache file. If the cache file does not exist, it prompts the user to create one. For each cached documentation, it copies the string to the clipboard and prompts the user to decide
-        what to do with it: paste and save, skip, or exit. If the user chooses to exit, any uncached items remain for later saving. The updated list of documentation strings is saved back to the cache file at the end of the operation.
+        Args:
+            build (bool): Indicates whether to rebuild the docstring cache.
 
         Raises:
-            IOError: If there is an error reading from or writing to the cache file.
+            FileNotFoundError: If the docstring cache file does not exist.
+
+        Notes:
+            - Copies each docstring to the clipboard and prompts for further action.
+            - Saves results not copied if the process is exited.
         """
+
+        if build:
+            self.build_doc_cache()
+
         path = Path(self.cache_dir, self.doc_cache_file_name)
         if not path.exists():
             print(
@@ -185,24 +219,6 @@ class DocManager:
 
         self._save_doc_cache(save_results)
 
-    def _save_doc_cache(self, save_results: list[SourceDoc]):
-        """
-        Saves a list of SourceDoc instances to a JSON file in the specified cache directory.
-
-            Args:
-                save_results (list[SourceDoc]): A list of SourceDoc instances to be saved.
-
-            Raises:
-                Exception: Raises an exception if writing to the file fails.
-
-            This method constructs the full path to the cache file, opens it in write mode, and serializes the provided SourceDoc instances using their model_dump() method.
-            The data is stored in a JSON format for later retrieval.
-        """
-        path = Path(self.cache_dir, self.doc_cache_file_name)
-        with open(path, "w", encoding="utf-8") as wf:
-            json.dump([x.model_dump() for x in save_results], wf)
-            return
-
     def make_docstrings(
         self, custom_instruction: str | None = None, profile: str | None = None
     ):
@@ -215,20 +231,17 @@ class DocManager:
         Usage:
             Run the script in the command line and when prompted, enter the name of the function or class. The generated docstring will be copied to the clipboard for easy pasting.
         """
-        if custom_instruction and profile:
-            print("You cannot provide a custom instruction when providing a profile")
-            return
 
         if profile:
             if profile not in DOC_PROFILES:
                 print("ERROR: Commit profile not found")
-                return
-            custom_instruction = DOC_PROFILES[profile]
+                sys.exit(1)
+            profile = DOC_PROFILES[profile]
 
         while True:
             func_name = input("Enter a function or class name: ")
             res = self._handle_doc_generation(
-                func_name, custom_instruction=custom_instruction
+                func_name, custom_instruction=custom_instruction, profile=profile
             )
             if not res:
                 continue
@@ -262,38 +275,155 @@ class DocManager:
             pyperclip.copy(doc_string)
             print(f"Copied to clipboard. Manually paste docstring @ {path}")
 
-    def _handle_module_doc_generation(
-        self, module_name: str, custom_instruction: str | None = None
-    ):
+    def iter_modules(self, full_build: bool = False):
         """
-        Generates documentation strings for a specified Python module based on its source code.
+        Iterate through cached module documentation and copy it to the clipboard.
+
+        This method loads the module docstring cache from a specified file, validates the cache contents, and allows the user to copy each module's docstring to their clipboard for manual usage.
+
+        Raises:
+          SystemExit: If the docstring cache file does not exist or is empty.
+
+        Usage:
+          1. Builds the module documentation cache using _build_module_doc_cache.
+          2. Reads from the cache file stored in the designated cache directory.
+          3. Allows copying of each module's docstring to the clipboard.
+        """
+
+        self._build_module_doc_cache(full_build=full_build)
+
+        path = Path(self.cache_dir, self.module_cache_file_name)
+        if not path.exists():
+            print("No Docstring cache found")
+            sys.exit(1)
+
+        with open(path, "r", encoding="utf-8") as rf:
+            content = json.load(rf)
+
+        results = [ModuleDocSave.model_validate(x) for x in content]
+
+        if not results:
+            print("No new modules found. All Good!")
+            sys.exit(0)
+
+        for result in results:
+            pyperclip.copy(result.doc)
+            print(f"Copied to clipboard. Manually paste docstring @ {result.path}")
+            do_pop = input("Press enter to continue...")
+
+    def _save_doc_cache(self, save_results: list[SourceDoc]):
+        """
+        Saves a list of SourceDoc instances to a JSON file in the specified cache directory.
 
             Args:
-                module_name (str): The name of the module for which to generate documentation.
-                custom_instruction (str | None): Additional instructions for customization, if any.
-
-            Returns:
-                tuple: A tuple containing the generated docstring and a link to the module's path in VSCode.
+                save_results (list[SourceDoc]): A list of SourceDoc instances to be saved.
 
             Raises:
-                Exception: If the module source code cannot be processed for documentation generation.
+                Exception: Raises an exception if writing to the file fails.
+
+            This method constructs the full path to the cache file, opens it in write mode, and serializes the provided SourceDoc instances using their model_dump() method.
+            The data is stored in a JSON format for later retrieval.
+        """
+        path = Path(self.cache_dir, self.doc_cache_file_name)
+        with open(path, "w", encoding="utf-8") as wf:
+            json.dump([x.model_dump() for x in save_results], wf)
+            return
+
+    def _build_module_doc_cache(self, full_build: bool = False):
+        """
+        Generate and cache documentation for modules.
+
+        Args:
+            full_build (bool): A flag indicating whether to perform a full build.
+
+        Returns:
+            None
+
+        Raises:
+            Exception: If an error occurs during documentation generation.
         """
 
-        similar_modules = [mod for mod in self.indexer.file_index if module_name in mod]
+        files_to_process: set[str] = set()
 
-        if not similar_modules:
-            print("Unable to locate module. Write using the full file path")
-            return sys.exit(1)
+        for file in self.indexer.file_index:
+            content = Path(file).read_text(encoding="utf-8")
+            if len(content) < 10:
+                continue
+            content_ast = ast.parse(content)
 
-        if len(similar_modules) > 1:
-            print("Found multiple elements. Please select the proper one:")
-            for idx, item in enumerate(similar_modules):
-                print(f"{idx:<4}:{item}...")
-            index = int(input("Type index: "))
+            if not content_ast.body:
+                continue
+            has_doc = (
+                isinstance(content_ast.body[0], ast.Expr)
+                and isinstance(content_ast.body[0].value, ast.Constant)
+                and len(content_ast.body[0].value.value) > 3
+            )
 
-            module_path = similar_modules[index]
-        else:
-            module_path = similar_modules[0]
+            if has_doc and not full_build:
+                continue
+            files_to_process.add(file)
+
+        save_candidates: list[ModuleDocSave] = []
+        with ThreadPoolExecutor(max_workers=20) as tpe:
+            futures = [
+                tpe.submit(self._handle_module_doc_generation, module_path=mod_path)
+                for mod_path in files_to_process
+            ]
+
+            for future in as_completed(futures):
+                try:
+                    res = future.result()
+                    save_candidates.append(ModuleDocSave(doc=res[0], path=res[1]))
+                except Exception as e:
+                    print(type(e), e)
+                    traceback.print_exc()
+
+        if not save_candidates:
+            print("No docs to save")
+            return
+
+        with open(
+            os.path.join(self.cache_dir, self.module_cache_file_name),
+            "w",
+            encoding="utf-8",
+        ) as wf:
+            json.dump([x.model_dump() for x in save_candidates], wf)
+
+    def _handle_module_doc_generation(
+        self,
+        module_name: str | None = None,
+        module_path: str | None = None,
+        custom_instruction: str | None = None,
+    ) -> tuple[str, str]:
+        """
+        Generates module documentation based on the supplied module name or path.
+
+        Args:
+            module_name (str | None): The name of the module to document. If None, the function tries to find the module based on the module_path.
+            module_path (str | None): The path to the module's source code. If None, it will attempt to locate the module using the module_name.
+            custom_instruction (str | None): Additional instructions for generating the documentation, if needed.
+
+        Returns:
+            tuple | None: A tuple containing the generated docstring and a VSCode link to the module file, or None if unable to generate documentation.
+        """
+        if not module_path:
+            similar_modules = [
+                mod for mod in self.indexer.file_index if module_name in mod
+            ]
+
+            if not similar_modules:
+                print("Unable to locate module. Write using the full file path")
+                return sys.exit(1)
+
+            if len(similar_modules) > 1:
+                print("Found multiple elements. Please select the proper one:")
+                for idx, item in enumerate(similar_modules):
+                    print(f"{idx:<4}:{item}...")
+                index = int(input("Type index: "))
+
+                module_path = similar_modules[index]
+            else:
+                module_path = similar_modules[0]
 
         path = Path(module_path)
         source_code = ast.parse(path.read_text(encoding="utf-8"))
@@ -332,21 +462,20 @@ class DocManager:
         object_def: SourceCode | None = None,
         pbar: tqdm | None = None,
         custom_instruction: str | None = None,
+        profile: str | None = None,
     ) -> SourceDoc | None:
         """
-        Generates a documentation string for a specified function or object.
+        _handle_doc_generation function generates documentation strings for specified functions or objects.
 
         Args:
-            function_name (str | None): The name of the function to document. Default is None.
-            object_def (SourceCode | None): An optional SourceCode object containing the definition.
-            pbar (tqdm | None): An optional progress bar for tracking status during generation.
+            function_name (str, optional): The name of the function for which documentation is to be generated.
+            object_def (SourceCode, optional): An optional SourceCode object containing the source details of the function.
+            pbar (tqdm, optional): A progress bar for tracking the operation, if applicable.
+            custom_instruction (str, optional): Any custom instructions for generating the documentation.
+            profile (str, optional): Profile information that may guide the documentation generation.
 
         Returns:
-            SourceDoc | None: A SourceDoc object containing the generated doc string and source code,
-                               or None if generation fails due to errors or missing source code.
-
-        Raises:
-            Exception: Prints error message if documentation generation fails.
+            SourceDoc | None: A SourceDoc object containing the generated documentation string and its source, or None if generation fails.
         """
 
         if pbar:
@@ -355,7 +484,9 @@ class DocManager:
         if not function_name and not object_def:
             raise ValueError("Please provide a function name or an object_def")
 
-        source_code = object_def or self._get_source_code_from_name(function_name)  # type: ignore #fmt:skip
+        source_code = object_def or NodeIndexer.get_source_code_from_name(
+            self.indexer.index, function_name
+        )
         if not source_code:
             print(
                 "ERROR: Unable to locate object in the index. Double check the name you entered."
@@ -372,6 +503,7 @@ class DocManager:
                 dependencies,
                 silence=bool(pbar),
                 custom_instruction=custom_instruction,
+                profile=profile,
             )
         except Exception as e:
             print(e)
@@ -385,36 +517,6 @@ class DocManager:
         doc_string = '"""\n' + doc_string + '\n"""'
         return SourceDoc(doc_string=doc_string, source=source_code)
 
-    def _get_source_code_from_name(self, obj_name: str) -> SourceCode | None:
-        """
-        Retrieves the source code associated with a specified object name from the index.
-
-        Args:
-            obj_name (str): The name of the object to retrieve the source code for.
-
-        Returns:
-            SourceCode | None: The source code of the object if found, or None if no object matches the name.
-
-        Raises:
-            ValueError: If multiple objects are found, prompts the user to specify which object to select.
-        """
-
-        func = list(self.indexer.index[obj_name])
-        if not func:
-            return None
-
-        if len(func) > 1:
-            print("Found multiple elements. Please select the proper one:")
-            for idx, item in enumerate(func):
-                print(f"{idx:<4}:{item.location}...")
-            index = int(input("Type index: "))
-
-            object_def = func[index]
-        else:
-            object_def = func[0]
-
-        return object_def
-
     def _generate_doc(
         self,
         func_name: str,
@@ -422,6 +524,7 @@ class DocManager:
         dependencies: list[str] | None,
         silence: bool = False,
         custom_instruction: str | None = None,
+        profile: str | None = None,
     ):
         """
         Generate a docstring for the specified function.
@@ -438,23 +541,10 @@ class DocManager:
         """
         if not silence:
             print(f"Generating docstrings for '{func_name}'")
-        client = OpenAI(timeout=30)
+        client = OpenAI(timeout=180)
         if not dependencies:
             dependencies = []
 
-        class Step(BaseModel):
-            """#pythion:ignore"""
-
-            why_does_this_object_exist: str | None = None
-            what_purpose_does_it_serve: str | None = None
-
-        class DocString(BaseModel):
-            """#pythion:ignore"""
-
-            steps: list[Step]
-            main_object_name: str
-            main_object_docstring: str
-
         messages = [
             {
                 "role": "system",
@@ -467,6 +557,13 @@ class DocManager:
                 "content": "Dependency Source code: " + "\n\n".join(dependencies),
             },
         ]
+        if profile:
+            messages.append(
+                {
+                    "role": "user",
+                    "content": profile,
+                }
+            )
         if custom_instruction:
             messages.append(
                 {
@@ -478,7 +575,7 @@ class DocManager:
         completion = client.beta.chat.completions.parse(
             model="gpt-4o-mini",
             messages=messages,  # type:ignore
-            response_format=DocString,
+            response_format=ObjDocString,
         )
 
         ai_repsonse = completion.choices[0].message
@@ -493,40 +590,28 @@ class DocManager:
         custom_instruction: str | None = None,
     ):
         """
-        Generates docstrings for a specified module.
+        Generates a docstring for a specified Python module.
 
-            Args:
-                module_name (str): The name of the module to document.
-                module_source_code (str): The source code of the module.
-                custom_instruction (str | None, optional): Additional instructions for docstring generation. Defaults to None.
+        Args:
+            module_name (str): The name of the module.
+            module_source_code (str): The source code of the module.
+            custom_instruction (str | None): Additional instructions for docstring generation.
 
-            Returns:
-                str | None: The generated docstring for the module, or None if generation fails.
+        Returns:
+            str | None: The generated docstring or None if generation fails.
         """
         print(f"Generating docstrings for module '{module_name}'")
-        client = OpenAI(timeout=30)
-
-        class Step(BaseModel):
-            """#pythion:ignore"""
-
-            why_does_this_module_exist: str | None = None
-            what_purpose_does_it_serve: str | None = None
-
-        class DocString(BaseModel):
-            """#pythion:ignore"""
-
-            steps: list[Step]
-            module_name: str
-            module_docstring: str
+        client = OpenAI(timeout=180)
 
         messages = [
             {
                 "role": "system",
-                "content": "You are a Python module docstring writer. Your task is to look at the module source code and write a doc string to put at the top of the file.\n\nThe format I want is Google Style. Format neatly with list items (if any). Keep documentation simple and minimal. Keep sentences short and use bullet points. Max sentence length: 100 characters",
+                "content": "You are a Python module docstring writer. Your task is to look at the module source code and write a doc string to put at the top of the file.\n\nThe format I want is Google Style. Format neatly with list items (if any). Keep documentation simple and minimal and do not do indent headings. Keep sentences short and use bullet points. Max sentence length: 100 characters",
             },
             {"role": "user", "content": "Module Name: " + module_name},
             {"role": "user", "content": "Module source code: " + module_source_code},
         ]
+
         if custom_instruction:
             messages.append(
                 {
@@ -538,7 +623,7 @@ class DocManager:
         completion = client.beta.chat.completions.parse(
             model="gpt-4o-mini",
             messages=messages,  # type:ignore
-            response_format=DocString,
+            response_format=ModuleDocString,
         )
 
         ai_repsonse = completion.choices[0].message
