@@ -67,6 +67,22 @@ class CallFinder(ast.NodeVisitor):
         """
         self.generic_visit(node)
 
+    def visit_AsyncFunctionDef(self, node: ast.AsyncFunctionDef):
+        """
+        Visits a class definition node in an Abstract Syntax Tree (AST).
+
+        This method is part of a visitor pattern for traversing AST nodes. It calls the
+        `generic_visit` method to handle the visit according to the AST structure.
+
+        Args:
+            node (ast.ClassDef): The class definition node to be visited.
+
+        Returns:
+            None: This method does not return a value but may modify the state of the
+        dynamic visitor depending on its implementation.
+        """
+        self.generic_visit(node)
+
     def visit_ClassDef(self, node: ast.ClassDef):
         """
         Visits a FunctionDef node in an Abstract Syntax Tree (AST).
@@ -138,7 +154,7 @@ class NodeTransformer(ast.NodeTransformer):
             whether a docstring was present and removed.
         """
         has_docstring = False
-        if not isinstance(node, ast.FunctionDef):
+        if not isinstance(node, (ast.FunctionDef, ast.AsyncFunctionDef)):
             return node, has_docstring
         if (
             node.body
@@ -207,6 +223,29 @@ class NodeTransformer(ast.NodeTransformer):
         )
         return node
 
+    def visit_AsyncFunctionDef(self, node: ast.FunctionDef) -> ast.FunctionDef:
+        """
+        Processes and cleans a FunctionDef AST node and indexes its details.
+
+        Args:
+            node (ast.FunctionDef): The AST node representing a function definition.
+
+        Returns:
+            ast.FunctionDef: The cleaned and processed function definition node.
+        """
+        clean_node, has_docstring = self.clean_function(deepcopy(node))
+        self.generic_visit(node)
+        self.index[clean_node.name].add(
+            SourceCode(
+                object_name=clean_node.name,
+                object_type="function",
+                file_path=self.current_path,
+                source_code=ast.unparse(clean_node),
+                has_docstring=has_docstring,
+            )
+        )
+        return node
+
     def visit_ClassDef(self, node: ast.ClassDef) -> ast.ClassDef:
         """
         Processes a class definition node in an Abstract Syntax Tree (AST).
@@ -255,7 +294,7 @@ class NodeIndexer:
         """"""
         self.root_dir = root_dir
         self.index: dict[str, set[SourceCode]] = defaultdict(set)
-        self.file_index: set = set()
+        self.file_index: set[str] = set()
         self.folders_to_ignore = [".venv", ".mypy_cache"]
         if folders_to_ignore:
             self.folders_to_ignore.extend(folders_to_ignore)
@@ -350,7 +389,7 @@ class NodeIndexer:
         call_finder.visit(node)
         if recursive:
             for call in deepcopy(call_names):
-                dep_node = self.index.get(call, set())
+                dep_node = self.index.get(call)
                 if not dep_node:
                     continue
                 for sub_node in dep_node:
@@ -368,15 +407,16 @@ class NodeIndexer:
 
     def _get_args(self, node: ast.FunctionDef) -> set[str] | None:
         """
-        Retrieve argument types from a function definition node.
+        Extracts argument types from a function definition node.
 
         Args:
-            node (ast.FunctionDef): The function definition node from which to extract
-                the argument types.
+            node (ast.FunctionDef): The function definition node from which to extract argument types.
 
         Returns:
-            set[str] | None: A set of argument type names if the node is a valid
-                function definition, or None if it is not.
+            set[str] | None: A set of argument type names if the node is a valid function definition, otherwise None.
+
+        Raises:
+            TypeError: If the input node is not an instance of ast.FunctionDef.
         """
         if not isinstance(node, ast.FunctionDef):
             return None
@@ -387,6 +427,12 @@ class NodeIndexer:
             if isinstance(arg.annotation, ast.BinOp):
                 sub_types = self._get_arg_from_binop(arg.annotation)
                 arg_types.update(sub_types)
+            if isinstance(arg.annotation, ast.Subscript):
+                if not isinstance(arg.annotation.slice, ast.Tuple):
+                    continue
+                for elt in arg.annotation.slice.elts:
+                    if isinstance(elt, ast.Name):
+                        arg_types.add(elt.id)
 
         return arg_types
 
@@ -445,7 +491,7 @@ class NodeIndexer:
                 node = obj
                 break
         else:
-            return None
+            raise ValueError("Invalid Object ID")
 
         arg_types: set[str] = set()
         node_src: ast.Module | ast.stmt | ast.FunctionDef | ast.ClassDef = ast.parse(
@@ -513,6 +559,36 @@ class NodeIndexer:
         for dup in duplicate_names:
             print(dup.location)
 
+    @staticmethod
+    def get_source_code_from_name(
+        index: dict[str, set[str]], obj_name: str
+    ) -> SourceCode | None:
+        """
+        Retrieves the source code of an object based on its name.
+
+        Args:
+            index (dict[str, set[str]]): A dictionary mapping object names to their definitions.
+            obj_name (str): The name of the object for which to retrieve the source code.
+
+        Returns:
+            SourceCode | None: The source code of the specified object or None if the object is not found.
+        """
+        func = list(index[obj_name])
+        if not func:
+            return None
+
+        if len(func) > 1:
+            print("Found multiple elements. Please select the proper one:")
+            for idx, item in enumerate(func):
+                print(f"{idx:<4}:{item.location}...")
+            index = int(input("Type index: "))
+
+            object_def = func[index]
+        else:
+            object_def = func[0]
+
+        return object_def
+
 
 if __name__ == "__main__":
     indexer = NodeIndexer(".")
